package net.sf.javaprinciples.resource;

import java.io.File;

public class FilesystemResourceCache implements ResourceCache
{
    private ResourceCreator resourceCreator;
    private ResourceMetadataDao resourceMetadataDao;
    private String resourceBeanName;
    private FileResolver fileResolver;

    /**
     * @param identifier Never null
     * @return true when the file is on the disk
     */
    @Override
    public boolean containsResource(ResourceIdentifier identifier)
    {
        return fileResolver.resolve(identifier) != null;
    }

    /**
     * @param identifier Never null
     * @return a resource if present
     */
    @Override
    public Resource getResource(ResourceIdentifier identifier)
    {
        File file = fileResolver.resolve(identifier);
        if (file == null || !file.exists())
        {
            return null;
        }

        FilesystemResource resource = new FilesystemResource();
        resource.setFile(file);

        File metadataFile = new File(file.getParent(), "metadata");
        if (metadataFile.exists())
        {
            ResourceMetadata resourceMetadata = resourceMetadataDao.fetch(metadataFile);
            resource.setMetadata(resourceMetadata);
        }
        return resource;
    }

    /**
     * Clear on disk cache.
     */
    public void clear()
    {
        throw new UnsupportedOperationException();
    }

    /**
     * Clear specific resource from on disk cache.
     * @param identifier The identifier of the resource to remove.
     */
    public void clear(ResourceIdentifier identifier)
    {
        throw new UnsupportedOperationException();
    }

    /**
     * @param identifier never null
     * @param resource never null
     */
    @Override
    public void putResource(ResourceIdentifier identifier, Resource resource)
    {
    }

    public void setFileResolver(FileResolver fileResolver)
    {
        this.fileResolver = fileResolver;
    }

    public void setResourceBeanName(String resourceBeanName)
    {
        this.resourceBeanName = resourceBeanName;
    }

    public String getResourceBeanName()
    {
        return resourceBeanName;
    }

    public void setResourceCreator(ResourceCreator resourceCreator)
    {
        this.resourceCreator = resourceCreator;
    }

    public ResourceCreator getResourceCreator()
    {
        return resourceCreator;
    }

    public void setResourceMetadataDao(ResourceMetadataDao resourceMetadataDao)
    {
        this.resourceMetadataDao = resourceMetadataDao;
    }
}
