package net.sf.javaprinciples.resource;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.text.ParseException;
import java.util.Map;

import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLEventWriter;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;
import org.codehaus.jettison.AbstractXMLOutputFactory;
import org.codehaus.jettison.mapped.Configuration;
import org.codehaus.jettison.mapped.MappedXMLOutputFactory;

/**
 * Formatter to format an XML result into JSON.
 * @author rvl
 */
public class JsonXmlResourceFormatter implements ResourceFormatter<String,String>
{
    private Map<Object, Object> namespaces;

    /**
     * @param resource Any XML content that can be converted, never null
     * @return The formatted content, never null
     */
    @Override
    public String formatResource(String resource)
    {
        try
        {
            return convertXmlToJson(resource);
        }
        catch (ParseException pe)
        {
//            Loggers.validation.error("Cannout convert XML to JSON {2}", "JXRF-01", pe.getMessage());
            return "{\"fault\":\"There was a problem retrieving the resource.\"}";
        }
    }

    @SuppressWarnings(value={"unchecked"})
    private String convertXmlToJson(String xmlToConvert) throws ParseException
    {
        if (xmlToConvert == null)
        {
            throw new ParseException("Unable to convert XML to JSON", 0);
        }
        try
        {
            String xmlWithoutBreaks = removeLineBreaks(xmlToConvert);

            byte[] xmlByteArray = xmlWithoutBreaks.getBytes();
            Configuration config = new Configuration();
            if (null != namespaces && !namespaces.isEmpty())
            {
                config.getXmlToJsonNamespaces().putAll(namespaces);
            }
            ByteArrayInputStream source = new ByteArrayInputStream(xmlByteArray);
            AbstractXMLOutputFactory writerFactory = new MappedXMLOutputFactory(config);
            ByteArrayOutputStream output = serialise(source, writerFactory);
            return new String(output.toByteArray());
        }
        catch (IOException e)
        {
            ParseException ex = new ParseException("Unable to convert XML to JSON", 0);
            ex.setStackTrace(e.getStackTrace());
            throw ex;
        }
    }

    private ByteArrayOutputStream serialise(InputStream source,
            AbstractXMLOutputFactory writerFactory)
            throws IOException
    {
        try
        {
            ByteArrayOutputStream output = new ByteArrayOutputStream();
            XMLInputFactory readerFactory = XMLInputFactory.newInstance();
            // create a StAX stream reader to move across the source
            XMLStreamReader streamReader = readerFactory.createXMLStreamReader(source);
            XMLEventReader eventReader = readerFactory.createXMLEventReader(streamReader);
            XMLEventWriter eventWriter = writerFactory.createXMLEventWriter(output);
            eventWriter.add(eventReader);
            eventWriter.close();
            return output;
        }
        catch (XMLStreamException ex)
        {
            IOException ioex = new IOException("Cannot serialize: " + source);
            ioex.initCause(ex);
            throw ioex;
        }
    }

    private String removeLineBreaks(String resource)
    {
        String escapedResource = resource.replace("\n", "")
            .replace("\r", "").replace("&#13;", "");
        return escapedResource;
    }

    public void setNamespaces(Map<Object, Object> namespaces)
    {
        this.namespaces = namespaces;
    }
}
