package net.sf.javaprinciples.resource.servlet;

import java.io.IOException;
import java.util.Calendar;
import java.util.GregorianCalendar;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;

/**
 * A Servlet Filter to log the access and execution time of a URL.
 */
public class ServletLoggingFilter implements Filter
{
    private FilterConfig config;

    @Override
    public void destroy()
    {
        config = null;
    }

    /**
     * @param request - ServletRequest
     * @param response - ServletResponse
     * @param chain - FilterChain
     * @throws IOException - exception
     * @throws ServletException - exception
     */
    @Override
    public void doFilter(ServletRequest request, ServletResponse response,
            FilterChain chain) throws IOException, ServletException
    {
        
        String uri = "";
        String name = "";
        if (request instanceof HttpServletRequest)
        {
            uri = ((HttpServletRequest) request).getRequestURI();
            name = ((HttpServletRequest) request).getSession().getServletContext().getServletContextName();
        }

        long before = System.currentTimeMillis();
        Calendar cal = new GregorianCalendar();
        
        String entry = cal.get(Calendar.HOUR_OF_DAY) + ":" + cal.get(Calendar.MINUTE) + ":" + cal.get(Calendar.SECOND);
        
        config.getServletContext().log(name + " URI accessed: " + uri + ": Entry Time: " + entry);
        chain.doFilter(request, response);
        long after = System.currentTimeMillis();
        
        config.getServletContext().log(name + " URI accessed: " + uri + ": Time taken to execute: "
            + (after - before) + "ms");
    }

    /**
     * @param config  FilterConfig.
     * @throws ServletException - exception.
     */
    @Override
    public void init(FilterConfig config) throws ServletException
    {
        this.config = config;
    }
}
